#!/usr/bin/env python3

from fenrirscreenreader.core.i18n import _


class command():
    def __init__(self):
        pass

    def initialize(self, environment):
        self.env = environment

    def shutdown(self):
        pass

    def getDescription(self):
        return "Apply the last tested voice from safe voice browser"

    def run(self):
        try:
            # Check if we have a tested voice
            if ('commandBuffer' not in self.env or
                'lastTestedModule' not in self.env['commandBuffer'] or
                    'lastTestedVoice' not in self.env['commandBuffer']):
                self.env['runtime']['outputManager'].presentText(
                    "No voice has been tested yet", interrupt=True)
                self.env['runtime']['outputManager'].presentText(
                    "Use voice browser first", interrupt=True)
                return

            module = self.env['commandBuffer']['lastTestedModule']
            voice = self.env['commandBuffer']['lastTestedVoice']

            self.env['runtime']['outputManager'].presentText(
                f"Applying {voice} from {module}", interrupt=True)

            # Apply to runtime settings only (temporary until saved)
            settingsManager = self.env['runtime']['settingsManager']

            # Store old values for safety
            oldDriver = settingsManager.getSetting('speech', 'driver')
            oldModule = settingsManager.getSetting('speech', 'module')
            oldVoice = settingsManager.getSetting('speech', 'voice')

            try:
                # Apply new settings to runtime only (use setSetting to update
                # settingArgDict)
                settingsManager.setSetting('speech', 'driver', 'speechdDriver')
                settingsManager.setSetting('speech', 'module', module)
                settingsManager.setSetting('speech', 'voice', voice)

                # Apply to speech driver instance directly
                if 'speechDriver' in self.env['runtime']:
                    speechDriver = self.env['runtime']['speechDriver']

                    # Set the module and voice on the driver instance
                    speechDriver.setModule(module)
                    speechDriver.setVoice(voice)

                    self.env['runtime']['outputManager'].presentText(
                        "Voice applied successfully!", interrupt=True)
                    self.env['runtime']['outputManager'].presentText(
                        "Use save settings to make permanent", interrupt=True)
                    self.env['runtime']['outputManager'].playSound('Accept')

            except Exception as e:
                # Revert on failure
                settingsManager.setSetting('speech', 'driver', oldDriver)
                settingsManager.setSetting('speech', 'module', oldModule)
                settingsManager.setSetting('speech', 'voice', oldVoice)

                self.env['runtime']['outputManager'].presentText(
                    f"Failed to apply voice, reverted: {str(e)}", interrupt=True)
                self.env['runtime']['outputManager'].playSound('Error')

        except Exception as e:
            self.env['runtime']['outputManager'].presentText(
                f"Apply voice error: {str(e)}", interrupt=True)
            self.env['runtime']['outputManager'].playSound('Error')

    def setCallback(self, callback):
        pass
