#!/usr/bin/env python3

from fenrirscreenreader.core.i18n import _

import os
import importlib.util

# Load base configuration class
_base_path = os.path.join(os.path.dirname(__file__), '..', 'config_base.py')
_spec = importlib.util.spec_from_file_location("config_base", _base_path)
_module = importlib.util.module_from_spec(_spec)
_spec.loader.exec_module(_module)
ConfigCommand = _module.ConfigCommand


class command(ConfigCommand):
    def __init__(self):
        super().__init__()

    def getDescription(self):
        return "Set punctuation verbosity level"

    def run(self):
        currentLevel = self.getSetting('general', 'punctuationLevel', 'some')

        # Present current level
        levelDescriptions = {
            'none': 'None - no punctuation spoken',
            'some': 'Some - basic punctuation only',
            'most': 'Most - detailed punctuation',
            'all': 'All - every punctuation mark'
        }

        currentDescription = levelDescriptions.get(currentLevel, 'Unknown')
        self.presentText(f"Current punctuation level: {currentDescription}")

        # Cycle through the four levels
        levels = ['none', 'some', 'most', 'all']
        try:
            currentIndex = levels.index(currentLevel)
            nextIndex = (currentIndex + 1) % len(levels)
            newLevel = levels[nextIndex]
        except ValueError:
            newLevel = 'some'  # Default to some

        success = self.setSetting('general', 'punctuationLevel', newLevel)

        if success:
            newDescription = levelDescriptions[newLevel]
            self.presentText(f"Punctuation level set to: {newDescription}")
            self.playSound('Accept')
        else:
            self.presentText("Failed to change punctuation level")
            self.playSound('Error')
