#!/usr/bin/env python3

from fenrirscreenreader.core.i18n import _

import os
import importlib.util

# Load base configuration class
_base_path = os.path.join(os.path.dirname(__file__), '..', 'config_base.py')
_spec = importlib.util.spec_from_file_location("config_base", _base_path)
_module = importlib.util.module_from_spec(_spec)
_spec.loader.exec_module(_module)
ConfigCommand = _module.ConfigCommand


class command(ConfigCommand):
    def __init__(self):
        super().__init__()

    def getDescription(self):
        return "Toggle debug mode"

    def run(self):
        currentLevel = self.getSetting('general', 'debugLevel', '0')

        # Present current debug level
        if currentLevel == '0':
            self.presentText("Current debug mode: disabled")
            newLevel = '1'
            stateText = "enabled"
        else:
            self.presentText(f"Current debug level: {currentLevel}")
            newLevel = '0'
            stateText = "disabled"

        success = self.setSetting('general', 'debugLevel', newLevel)

        if success:
            self.presentText(f"Debug mode {stateText}")
            if newLevel != '0':
                debugMode = self.getSetting('general', 'debugMode', 'File')
                if debugMode == 'File':
                    self.presentText(
                        "Debug output will be written to log file")
                else:
                    self.presentText("Debug output will be printed to console")
            self.playSound('Accept')
        else:
            self.presentText("Failed to change debug mode")
            self.playSound('Error')
