#!/usr/bin/env python3

from fenrirscreenreader.core.i18n import _

import os
import importlib.util

# Load base configuration class
_base_path = os.path.join(os.path.dirname(__file__), '..', 'config_base.py')
_spec = importlib.util.spec_from_file_location("config_base", _base_path)
_module = importlib.util.module_from_spec(_spec)
_spec.loader.exec_module(_module)
ConfigCommand = _module.ConfigCommand


class command(ConfigCommand):
    def __init__(self):
        super().__init__()

    def getDescription(self):
        return "Set character echo mode"

    def run(self):
        currentMode = self.getSetting('keyboard', 'charEchoMode', '1')

        # Present current mode
        modeDescriptions = {
            '0': 'None - no character echo',
            '1': 'Always - echo all typed characters',
            '2': 'Caps Lock - echo only when caps lock is on'
        }

        currentDescription = modeDescriptions.get(currentMode, 'Unknown')
        self.presentText(f"Current character echo mode: {currentDescription}")

        # Cycle through the three modes
        modes = ['0', '1', '2']
        try:
            currentIndex = modes.index(currentMode)
            nextIndex = (currentIndex + 1) % len(modes)
            newMode = modes[nextIndex]
        except ValueError:
            newMode = '1'  # Default to always

        success = self.setSetting('keyboard', 'charEchoMode', newMode)

        if success:
            newDescription = modeDescriptions[newMode]
            self.presentText(f"Character echo mode set to: {newDescription}")
            self.playSound('Accept')
        else:
            self.presentText("Failed to change character echo mode")
            self.playSound('Error')
