#!/usr/bin/env python3

from fenrirscreenreader.core.i18n import _

import os
import importlib.util

# Load base configuration class
_base_path = os.path.join(os.path.dirname(__file__), '..', 'config_base.py')
_spec = importlib.util.spec_from_file_location("config_base", _base_path)
_module = importlib.util.module_from_spec(_spec)
_spec.loader.exec_module(_module)
ConfigCommand = _module.ConfigCommand


class command(ConfigCommand):
    def __init__(self):
        super().__init__()

    def getDescription(self):
        return "Toggle exclusive keyboard access"

    def run(self):
        currentState = self.getBooleanSetting('keyboard', 'grabDevices', True)
        newState = self.toggleBooleanSetting('keyboard', 'grabDevices')

        if newState != currentState:
            stateText = "enabled" if newState else "disabled"
            self.presentText(f"Exclusive keyboard access {stateText}")
            if newState:
                self.presentText(
                    "Fenrir will have exclusive control of keyboard input")
            else:
                self.presentText(
                    "Fenrir will share keyboard input with other applications")
            self.presentText(
                "Please restart Fenrir for this change to take effect")
            self.playSound('Accept')
        else:
            self.presentText("Failed to change keyboard grab setting")
            self.playSound('Error')
