#!/usr/bin/env python3

from fenrirscreenreader.core.i18n import _

import os
import importlib.util

# Load base configuration class
_base_path = os.path.join(os.path.dirname(__file__), '..', 'config_base.py')
_spec = importlib.util.spec_from_file_location("config_base", _base_path)
_module = importlib.util.module_from_spec(_spec)
_spec.loader.exec_module(_module)
ConfigCommand = _module.ConfigCommand


class command(ConfigCommand):
    def __init__(self):
        super().__init__()

    def getDescription(self):
        return "Toggle word echo when pressing space"

    def run(self):
        currentState = self.getBooleanSetting('keyboard', 'wordEcho', False)
        newState = self.toggleBooleanSetting('keyboard', 'wordEcho')

        if newState != currentState:
            stateText = "enabled" if newState else "disabled"
            self.presentText(f"Word echo {stateText}")
            if newState:
                self.presentText("Words will be spoken when you press space")
            else:
                self.presentText(
                    "Words will not be spoken when you press space")
            self.playSound('Accept')
        else:
            self.presentText("Failed to change word echo setting")
            self.playSound('Error')
