#!/usr/bin/env python3

from fenrirscreenreader.core.i18n import _

import os
import importlib.util

# Load base configuration class
_base_path = os.path.join(os.path.dirname(__file__), '..', 'config_base.py')
_spec = importlib.util.spec_from_file_location("config_base", _base_path)
_module = importlib.util.module_from_spec(_spec)
_spec.loader.exec_module(_module)
ConfigCommand = _module.ConfigCommand


class command(ConfigCommand):
    def __init__(self):
        super().__init__()

    def getDescription(self):
        return "Save current configuration to file"

    def run(self):
        self.presentText("Saving current configuration...")

        try:
            # Force reload and save of current configuration
            self.reloadConfig()

            # Write the configuration file
            with open(self.settingsFile, 'w') as configfile:
                self.config.write(configfile)

            self.presentText("Configuration saved successfully")
            self.playSound('Accept')

        except Exception as e:
            self.presentText(f"Failed to save configuration: {str(e)}")
            self.playSound('Error')
