#!/usr/bin/env python3

from fenrirscreenreader.core.i18n import _

import configparser


class command():
    def __init__(self):
        pass

    def initialize(self, environment):
        self.env = environment

    def shutdown(self):
        pass

    def getDescription(self):
        return "Show temporary changes not yet saved to file"

    def run(self):
        self.env['runtime']['outputManager'].presentText(
            "Checking for unsaved changes...", interrupt=True)

        try:
            # Read the current config file
            settingsFile = self.env['runtime']['settingsManager'].settingsFile
            fileConfig = configparser.ConfigParser(interpolation=None)
            fileConfig.read(settingsFile)

            # Compare with runtime settings
            runtimeSettings = self.env['runtime']['settingsManager'].settings

            changes = []

            # Check speech settings specifically
            speechSections = [
                'speech',
                'sound',
                'keyboard',
                'screen',
                'general']

            for section in speechSections:
                if section in runtimeSettings and section in fileConfig:
                    for option in runtimeSettings[section]:
                        runtimeValue = runtimeSettings[section][option]
                        try:
                            fileValue = fileConfig[section][option]
                        except Exception as e:
                            fileValue = ""

                        if runtimeValue != fileValue:
                            changes.append(
                                f"{section}.{option}: {fileValue} → {runtimeValue}")

            if changes:
                self.env['runtime']['outputManager'].presentText(
                    f"Found {len(changes)} unsaved changes:", interrupt=True)
                for change in changes[:5]:  # Limit to first 5 changes
                    self.env['runtime']['outputManager'].presentText(
                        change, interrupt=True)

                if len(changes) > 5:
                    self.env['runtime']['outputManager'].presentText(
                        f"... and {len(changes) - 5} more changes", interrupt=True)

                self.env['runtime']['outputManager'].presentText(
                    "Use save command to make these changes permanent", interrupt=True)
                self.env['runtime']['outputManager'].playSound('Accept')
            else:
                self.env['runtime']['outputManager'].presentText(
                    "No unsaved changes found", interrupt=True)
                self.env['runtime']['outputManager'].playSound('Cancel')

        except Exception as e:
            self.env['runtime']['outputManager'].presentText(
                f"Error checking for changes: {str(e)}", interrupt=True)
            self.env['runtime']['outputManager'].playSound('Error')

    def setCallback(self, callback):
        pass
