#!/usr/bin/env python3

from fenrirscreenreader.core.i18n import _

import os
import importlib.util

# Load base configuration class
_base_path = os.path.join(os.path.dirname(__file__), '..', 'config_base.py')
_spec = importlib.util.spec_from_file_location("config_base", _base_path)
_module = importlib.util.module_from_spec(_spec)
_spec.loader.exec_module(_module)
ConfigCommand = _module.ConfigCommand


class command(ConfigCommand):
    def __init__(self):
        super().__init__()

    def getDescription(self):
        return "Select screen driver"

    def run(self):
        currentDriver = self.getSetting('screen', 'driver', 'vcsaDriver')

        # Present current driver
        driverDescriptions = {
            'vcsaDriver': 'VCSA Driver - Linux TTY console access',
            'ptyDriver': 'PTY Driver - terminal emulation',
            'dummyDriver': 'Dummy Driver - for testing only',
            'debugDriver': 'Debug Driver - development/debugging'
        }

        currentDescription = driverDescriptions.get(
            currentDriver, 'Unknown driver')
        self.presentText(f"Current screen driver: {currentDescription}")

        # Cycle through the available drivers
        drivers = ['vcsaDriver', 'ptyDriver', 'dummyDriver', 'debugDriver']
        try:
            currentIndex = drivers.index(currentDriver)
            nextIndex = (currentIndex + 1) % len(drivers)
            newDriver = drivers[nextIndex]
        except ValueError:
            newDriver = 'vcsaDriver'  # Default to VCSA

        success = self.setSetting('screen', 'driver', newDriver)

        if success:
            newDescription = driverDescriptions[newDriver]
            self.presentText(f"Screen driver changed to: {newDescription}")
            self.presentText(
                "Please restart Fenrir for this change to take effect.")
            self.playSound('Accept')
        else:
            self.presentText("Failed to change screen driver")
            self.playSound('Error')
