#!/usr/bin/env python3

from fenrirscreenreader.core.i18n import _

import os
import importlib.util

# Load base configuration class
_base_path = os.path.join(os.path.dirname(__file__), '..', 'config_base.py')
_spec = importlib.util.spec_from_file_location("config_base", _base_path)
_module = importlib.util.module_from_spec(_spec)
_spec.loader.exec_module(_module)
ConfigCommand = _module.ConfigCommand


class command(ConfigCommand):
    def __init__(self):
        super().__init__()

    def getDescription(self):
        return "Set screen text encoding"

    def run(self):
        currentEncoding = self.getSetting('screen', 'encoding', 'auto')

        # Present current encoding
        self.presentText(f"Current screen encoding: {currentEncoding}")

        # Cycle through available encodings
        encodings = ['auto', 'utf-8', 'cp1252', 'iso-8859-1']
        try:
            currentIndex = encodings.index(currentEncoding)
            nextIndex = (currentIndex + 1) % len(encodings)
            newEncoding = encodings[nextIndex]
        except ValueError:
            newEncoding = 'auto'  # Default to auto

        success = self.setSetting('screen', 'encoding', newEncoding)

        if success:
            self.presentText(f"Screen encoding set to: {newEncoding}")
            if newEncoding == 'auto':
                self.presentText(
                    "Fenrir will automatically detect text encoding")
            else:
                self.presentText(f"Fenrir will use {newEncoding} encoding")
            self.playSound('Accept')
        else:
            self.presentText("Failed to change screen encoding")
            self.playSound('Error')
