#!/usr/bin/env python3

from fenrirscreenreader.core.i18n import _

import os
import importlib.util

# Load base configuration class
_base_path = os.path.join(os.path.dirname(__file__), '..', 'config_base.py')
_spec = importlib.util.spec_from_file_location("config_base", _base_path)
_module = importlib.util.module_from_spec(_spec)
_spec.loader.exec_module(_module)
ConfigCommand = _module.ConfigCommand


class command(ConfigCommand):
    def __init__(self):
        super().__init__()

    def getDescription(self):
        return "Adjust sound volume"

    def run(self):
        currentVolume = self.getFloatSetting('sound', 'volume', 0.7)

        # Present current volume
        volumePercent = int(currentVolume * 100)
        self.presentText(f"Current sound volume: {volumePercent} percent")

        # Adjust volume by 10%
        newVolume = self.adjustFloatSetting('sound', 'volume', 0.1, 0.0, 1.0)

        if newVolume != currentVolume:
            newPercent = int(newVolume * 100)
            self.presentText(
                f"Sound volume set to {newPercent} percent",
                interrupt=False)
            self.playSound('Accept')
        else:
            # If we hit the maximum, try decreasing instead
            newVolume = self.adjustFloatSetting(
                'sound', 'volume', -0.1, 0.0, 1.0)
            if newVolume != currentVolume:
                newPercent = int(newVolume * 100)
                self.presentText(
                    f"Sound volume set to {newPercent} percent",
                    interrupt=False)
                self.playSound('Accept')
            else:
                self.presentText("Sound volume unchanged", interrupt=False)
                self.playSound('Cancel')
