#!/usr/bin/env python3

from fenrirscreenreader.core.i18n import _

import os
import importlib.util

# Load base configuration class
_base_path = os.path.join(os.path.dirname(__file__), '..', 'config_base.py')
_spec = importlib.util.spec_from_file_location("config_base", _base_path)
_module = importlib.util.module_from_spec(_spec)
_spec.loader.exec_module(_module)
ConfigCommand = _module.ConfigCommand


class command(ConfigCommand):
    def __init__(self):
        super().__init__()

    def getDescription(self):
        return "Select sound driver (genericDriver or gstreamerDriver)"

    def run(self):
        currentDriver = self.getSetting('sound', 'driver', 'genericDriver')

        # Present current driver
        self.presentText(f"Current sound driver: {currentDriver}")

        # Toggle between the two available drivers
        if currentDriver == 'genericDriver':
            newDriver = 'gstreamerDriver'
            driverDescription = "GStreamer Driver - advanced multimedia framework"
        else:
            newDriver = 'genericDriver'
            driverDescription = "Generic Driver - uses SOX for audio playback"

        success = self.setSetting('sound', 'driver', newDriver)

        if success:
            self.presentText(
                f"Sound driver changed to {newDriver}. {driverDescription}")
            self.presentText(
                "Please restart Fenrir for this change to take effect.")
            self.playSound('Accept')
        else:
            self.presentText("Failed to change sound driver")
            self.playSound('Error')
