#!/usr/bin/env python3

from fenrirscreenreader.core.i18n import _

import os
import importlib.util

# Load base configuration class
_base_path = os.path.join(os.path.dirname(__file__), '..', 'config_base.py')
_spec = importlib.util.spec_from_file_location("config_base", _base_path)
_module = importlib.util.module_from_spec(_spec)
_spec.loader.exec_module(_module)
ConfigCommand = _module.ConfigCommand


class command(ConfigCommand):
    def __init__(self):
        super().__init__()

    def getDescription(self):
        return "Toggle sound output on or off"

    def run(self):
        currentState = self.getBooleanSetting('sound', 'enabled', True)
        newState = self.toggleBooleanSetting('sound', 'enabled')

        if newState != currentState:
            stateText = "enabled" if newState else "disabled"
            self.presentText(f"Sound {stateText}")
            # Only play sound if we enabled sound
            if newState:
                self.playSound('Accept')
        else:
            self.presentText("Failed to change sound setting")
