#!/usr/bin/env python3

from fenrirscreenreader.core.i18n import _


class command():
    def __init__(self):
        pass

    def initialize(self, environment):
        self.env = environment

    def shutdown(self):
        pass

    def getDescription(self):
        return "Adjust speech rate (temporary - use save to make permanent)"

    def run(self):
        try:
            # Get current rate from runtime settings (may be temporary)
            settingsManager = self.env['runtime']['settingsManager']
            currentRate = float(settingsManager.getSetting('speech', 'rate'))
        except Exception as e:
            currentRate = 0.5

        # Present current rate
        ratePercent = int(currentRate * 100)
        self.env['runtime']['outputManager'].presentText(
            f"Current speech rate: {ratePercent} percent", interrupt=True)

        # Increase by 10%, wrap around if at max
        newRate = currentRate + 0.1
        if newRate > 1.0:
            newRate = 0.1  # Wrap to minimum

        # Apply ONLY to runtime - this is temporary until saved
        settingsManager = self.env['runtime']['settingsManager']
        settingsManager.settings['speech']['rate'] = str(newRate)

        # Apply to speech system immediately for this session
        if 'speechDriver' in self.env['runtime']:
            try:
                self.env['runtime']['speechDriver'].setRate(newRate)
            except Exception as e:
                print(
                    f'adjust_speech_rate setRate: Error setting speech rate: {
                        str(e)}')

        newPercent = int(newRate * 100)
        self.env['runtime']['outputManager'].presentText(
            f"Speech rate temporarily set to {newPercent} percent",
            interrupt=False,
            flush=False)
        self.env['runtime']['outputManager'].presentText(
            "Use save command to make permanent", interrupt=False, flush=False)

    def setCallback(self, callback):
        pass
