#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributers.

from fenrirscreenreader.core import debug
from fenrirscreenreader.core.remoteDriver import remoteDriver
from fenrirscreenreader.core.eventData import fenrirEventType

import select
import socket
import os
import os.path


class driver(remoteDriver):
    def __init__(self):
        remoteDriver.__init__(self)

    def initialize(self, environment):
        self.env = environment
        self.env['runtime']['processManager'].addCustomEventThread(
            self.watchDog, multiprocess=True)

    def watchDog(self, active, eventQueue):
        # echo "command say this is a test" | socat -
        # UNIX-CLIENT:/tmp/fenrirscreenreader-deamon.sock
        socketFile = ''
        try:
            socketFile = self.env['runtime']['settingsManager'].getSetting(
                'remote', 'socketFile')
        except Exception as e:
            self.env['runtime']['debug'].writeDebugOut(
                'unixDriver watchDog: Error getting socket file setting: ' + str(e),
                debug.debugLevel.ERROR)
        if socketFile == '':
            if self.env['runtime']['settingsManager'].getSetting(
                    'screen', 'driver') == 'vcsaDriver':
                socketFile = '/tmp/fenrirscreenreader-deamon.sock'
            else:
                socketFile = '/tmp/fenrirscreenreader-' + \
                    str(os.getppid()) + '.sock'
        if os.path.exists(socketFile):
            os.unlink(socketFile)
        self.fenrirSock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        self.fenrirSock.bind(socketFile)
        os.chmod(socketFile, 0o666)  # Allow all users to read/write
        self.fenrirSock.listen(1)
        while active.value:
            # Check if the client is still connected and if data is available:
            try:
                r, _, _ = select.select([self.fenrirSock], [], [], 0.8)
            except select.error:
                break
            if r == []:
                continue
            if self.fenrirSock in r:
                client_sock, client_addr = self.fenrirSock.accept()
                # Ensure client socket is always closed to prevent resource
                # leaks
                try:
                    try:
                        rawdata = client_sock.recv(8129)
                    except Exception as e:
                        self.env['runtime']['debug'].writeDebugOut(
                            'unixDriver watchDog: Error receiving data from client: ' + str(e),
                            debug.debugLevel.ERROR)
                        rawdata = b''  # Set default empty data if recv fails

                    try:
                        data = rawdata.decode("utf-8").rstrip().lstrip()
                        eventQueue.put({"Type": fenrirEventType.RemoteIncomming,
                                        "Data": data
                                        })
                    except Exception as e:
                        self.env['runtime']['debug'].writeDebugOut(
                            'unixDriver watchDog: Error decoding/queuing data: ' + str(e),
                            debug.debugLevel.ERROR)
                finally:
                    # Always close client socket, even if data processing fails
                    try:
                        client_sock.close()
                    except Exception as e:
                        self.env['runtime']['debug'].writeDebugOut(
                            'unixDriver watchDog: Error closing client socket: ' + str(e),
                            debug.debugLevel.ERROR)
        if self.fenrirSock:
            self.fenrirSock.close()
            self.fenrirSock = None
        if os.path.exists(socketFile):
            os.unlink(socketFile)
